from dataclasses import dataclass
from typing import Type, TypeVar

from .property import Property

T = TypeVar("T", bound="Email")


@dataclass
class Email(Property):
    """Email class

    ex.
    {'id': 'Io%7C%3A', 'type': 'email', 'email': 'sample@example.com'}
    """

    value: str

    def __init__(
        self,
        name: str,
        value: str = "",
        id: str | None = None,  # noqa: A002
    ) -> None:
        self.name = name
        self.value = value
        self.id = id

    @classmethod
    def of(cls: Type[T], key: str, param: dict) -> T:
        value = param.get("email")
        if value is not None and not isinstance(value, str):
            raise ValueError(f"email must be str, but got {type(value)}")
        return cls(id=param["id"], name=key, value=value or "")

    @classmethod
    def from_email(cls: Type[T], email: str, name: str | None = None) -> T:
        return cls(name=name or cls.PROP_NAME, value=email)

    @classmethod
    def empty(cls: Type[T], name: str | None = None) -> T:
        return cls(name=name or cls.PROP_NAME, value="")

    @property
    def type(self) -> str:
        return "email"

    def __dict__(self) -> dict:
        result = {
            "type": self.type,
            "email": None if self.value == "" else self.value,
        }
        if self.id is not None:
            result["id"] = self.id
        return {
            self.name: result,
        }

    def value_for_filter(self) -> str:
        raise NotImplementedError



from dataclasses import dataclass
from typing import Type, TypeVar

from .property import Property

T = TypeVar("T", bound="People")


@dataclass
class People(Property):
    """
        ユーザプロパティ

        ex.{'id': '%5Eo%5Dy', 'type': 'people', 'people': [{'object': 'user', 'id': '
    6f877603-07b9-4467-b811-31706eeb640c', 'name': 'Akira Kobori', 'avatar_url': 'https://s3-us-west-2.amazonaws.com/public.notion-static.c
    om/1f3a184f-88f3-4048-bb77-3bef8ff805d8/kobori_akira_blackline.png', 'type': 'person', 'person': {'email': 'private.beats@gmail.com'}}]
    }
    """

    def __init__(self, name: str, id: str | None = None, people_list: list | None = None) -> None:  # noqa: A002, FBT001
        self.name = name
        self.id = id
        self.people_list = people_list

    @classmethod
    def of(cls: Type[T], name: str, param: dict) -> T:
        id = param["id"]
        people_list = param["people"]
        return cls(
            name=name,
            id=id,
            people_list=people_list,
        )

    @property
    def type(self) -> str:
        return "people"

    def __dict__(self) -> dict:
        result = {
            "type": self.type,
            self.type: self.people_list,
        }
        if self.id is not None:
            result["id"] = self.id
        return {self.name: result}

    def value_for_filter(self) -> str:
        raise NotImplementedError



from dataclasses import dataclass
from typing import Type, TypeVar

from .property import Property

T = TypeVar("T", bound="Button")


@dataclass
class Button(Property):
    id: str
    name: str
    type: str = "button"

    @classmethod
    def of(cls: Type[T], key: str, property: dict) -> T:
        return cls(id=property["id"], name=key)

    def value_for_filter(self) -> str:
        raise NotImplementedError

    def __dict__(self) -> dict:
        return {
            self.name: {
                "id": self.id,
                "type": self.type,
                "button": {},
            },
        }



from dataclasses import dataclass
from typing import Any, Type, TypeVar

from .property import Property

T = TypeVar("T", bound="Relation")


@dataclass
class Relation(Property):
    id_list: list[str]
    text_list: list[str]  # NOTE: Notionのデータとしては扱わない。id_listに変換するために必要になることが多いため
    type: str = "relation"
    has_more: bool = False

    TYPE = "relation"

    def __init__(  # noqa: PLR0913
        self,
        name: str,
        id: str | None = None,  # noqa: A002
        id_list: list[str] | None = None,
        text_list: list[str] | None = None,
        has_more: bool | None = None,
    ) -> None:
        self.name = name
        self.id = id
        self.id_list = id_list or []
        self.text_list = text_list or []
        self.has_more = bool(has_more)

    def is_unconverted_id_list(self) -> bool:
        """text_listがあるがid_listがない場合にTrueを返す"""
        return len(self.text_list) > 0 and len(self.id_list) == 0

    @classmethod
    def of(cls: Type[T], name: str, property: dict[str, Any]) -> T:
        id_list = [r["id"] for r in property["relation"]]
        return cls(name=name, id_list=id_list, has_more=property["has_more"])

    @classmethod
    def from_id_list(cls: Type[T], id_list: list[str], name: str | None = None) -> T:
        return cls(
            name=name or cls.PROP_NAME,
            id_list=id_list,
        )

    @classmethod
    def from_id(cls: Type[T], id: str, name: str | None = None) -> T:
        return cls.from_id_list(name=name or cls.PROP_NAME, id_list=[id])

    def append(self, id: str) -> None:
        self.id_list.append(id)

    def __dict__(self) -> dict:
        result = {
            "type": self.type,
            "relation": [
                {
                    "id": id,
                }
                for id in self.id_list  # noqa: A001
            ],
            "has_more": self.has_more,
        }
        if self.id is not None:
            result["relation"]["id"] = self.id
        return {
            self.name: result,
        }

    def value_for_filter(self) -> str:
        raise NotImplementedError



from dataclasses import dataclass
from typing import Type, TypeVar

from .property import Property

T = TypeVar("T", bound="Files")


@dataclass(frozen=True)
class File:
    """File class"""

    name: str
    url: str
    expired_at: str | None = None

    @staticmethod
    def of(param: dict) -> "File":
        name = param["name"]
        type_ = param["type"]
        url = param[type_]["url"]
        expired_at = param[type_]["expiry_time"] if type_ == "file" else None
        return File(
            name=name,
            url=url,
            expired_at=expired_at,
        )


@dataclass
class Files(Property):
    """Files class

    ex.
    {'id': '%7BjJx', 'type': 'files', 'files': []}
    """

    _files: list

    def __init__(
        self,
        name: str,
        files: list = [],
        id: str | None = None,  # noqa: A002
    ) -> None:
        self.name = name
        self._files = files
        self.id = id

    @classmethod
    def of(cls: Type[T], key: str, param: dict) -> T:
        return cls(
            id=param["id"],
            name=key,
            files=param["files"],
        )

    @property
    def value(self) -> list[File]:
        return [File.of(param) for param in self._files]

    @property
    def type(self) -> str:
        return "files"

    def __dict__(self) -> dict:
        raise NotImplementedError("this dict method must not be called")

    def value_for_filter(self) -> str:
        raise NotImplementedError



from .block import Block
from .block_type import BlockType
from .video import Video
from .paragraph import Paragraph
from .quote import Quote
from .block_factory import BlockFactory
from .heading import Heading
from .divider import Divider
from .bulleted_list_item import BulletedListItem
from .embed import Embed
from .bookmark import Bookmark
from .code import Code
from .child_database import ChildDatabase
from .to_do import ToDo
from .callout import Callout
from .child_page import ChildPage
from .column_list import ColumnList
from .image import Image
from .numbered_list_item import NumberedListItem
from .table import Table

__all__ = [
    "Block",
    "BlockType",
    "Video",
    "Paragraph",
    "Quote",
    "BlockFactory",
    "Heading",
    "Divider",
    "BulletedListItem",
    "Embed",
    "Bookmark",
    "Code",
    "ChildDatabase",
    "ToDo",
    "Callout",
    "ChildPage",
    "ColumnList",
    "Image",
    "NumberedListItem",
    "Table",
]



from dataclasses import dataclass, field
from datetime import datetime, timedelta
from typing import Type, TypeVar, cast

from .datetime_utils import JST
from .properties.files import Files
from .property_translator import PropertyTranslator
from .base_operator import BaseOperator
from .block.block import Block
from .page.page_id import PageId
from .properties.checkbox import Checkbox
from .properties.cover import Cover
from .properties.date import Date
from .properties.email import Email
from .properties.formula import Formula
from .properties.icon import Icon
from .properties.multi_select import MultiSelect
from .properties.number import Number
from .properties.phone_number import PhoneNumber
from .properties.properties import Properties
from .properties.property import Property
from .properties.relation import Relation
from .properties.rollup import Rollup
from .properties.select import Select
from .properties.status import Status
from .properties.text import Text
from .properties.title import Title
from .properties.unique_id import UniqueId
from .properties.url import Url

T = TypeVar("T", bound="BasePage")

P = TypeVar("P", bound=Property)


class NotCreatedError(Exception):
    pass


class NotFoundPropertyError(Exception):
    def __init__(self, class_name: str, prop_name: str):
        super().__init__(f"{class_name} property not found. name: {prop_name}")


@dataclass
class BasePage:
    properties: Properties
    block_children: list[Block] = field(default_factory=list)
    id_: str | None = None
    url_: str | None = None
    created_time: datetime | None = None
    last_edited_time: datetime | None = None
    _created_by: BaseOperator | None = None
    _last_edited_by: BaseOperator | None = None
    cover: Cover | None = None
    icon: Icon | None = None
    archived: bool | None = False
    parent: dict | None = None
    object = "page"
    DATABASE_ID: str = "database_id"  # must be set in subclass

    @classmethod
    def create(
        cls: Type[T],
        properties: list[Property] | None = None,
        blocks: list[Block] | None = None,
        cover: Cover | None = None,
        icon: Icon | None = None,
    ) -> T:
        return cls(
            id_=None,
            url_=None,
            created_time=None,
            last_edited_time=None,
            _created_by=None,
            _last_edited_by=None,
            properties=Properties(values=properties or []),
            cover=cover,
            icon=icon,
            archived=False,
            parent=None,
            block_children=blocks or [],
        )

    def get_slack_text_in_block_children(self) -> str:
        # FIXME: block_childrenをBlocks型にしたうえで、メソッドをBlocksに移動する
        if not self.block_children or len(self.block_children) == 0:
            return ""
        return "\n".join([block.to_slack_text() for block in self.block_children])

    def get_title(self) -> Title:
        return self.properties.get_title()

    def get_title_text(self) -> str:
        return self.get_title().rich_text.to_plain_text()

    @property
    def created_at(self) -> datetime:
        if self.created_time is None:
            raise NotCreatedError("created_at is None.")
        return self.created_time

    @property
    def updated_at(self) -> datetime:
        if self.last_edited_time is None:
            raise NotCreatedError("created_at is None.")
        return self.last_edited_time

    def get_prop(self, instance_class: Type[P]) -> P:
        parent_class = self.__get_parent_class(instance_class)
        if parent_class not in [
            Checkbox,
            Date,
            Email,
            MultiSelect,
            Number,
            PhoneNumber,
            Relation,
            Select,
            Status,
            Text,
            Title,
            Url,
        ]:
            error_message = "instance_class must be one of the following classes: Checkbox, Date, Email, MultiSelect, \
                Number, PhoneNumber, Relation, Select, Status, Text, Title, Url"
            raise ValueError(error_message)
        result = self.properties.get_property(name=instance_class.PROP_NAME, instance_class=parent_class)
        if result is None:
            raise NotFoundPropertyError(class_name=instance_class.__name__, prop_name=instance_class.PROP_NAME)
        return cast(P, result)

    def __get_parent_class(self, instance_class: Type[P]) -> Type[P]:
        parent_classes = instance_class.__bases__
        if not parent_classes:
            return instance_class
        return parent_classes[0]

    def set_prop(self, value: Property) -> None:
        self.properties = self.properties.append_property(value)

    def get_status(self, name: str) -> Status:
        return self._get_property(name=name, instance_class=Status)  # type: ignore

    def get_text(self, name: str) -> Text:
        return self._get_property(name=name, instance_class=Text)  # type: ignore

    def get_date(self, name: str) -> Date:
        return self._get_property(name=name, instance_class=Date)  # type: ignore

    def get_select(self, name: str) -> Select:
        return self._get_property(name=name, instance_class=Select)  # type: ignore

    def get_multi_select(self, name: str) -> MultiSelect:
        return self._get_property(name=name, instance_class=MultiSelect)  # type: ignore

    def get_relation(self, name: str) -> Relation:
        return self._get_property(name=name, instance_class=Relation)  # type: ignore

    def get_checkbox(self, name: str) -> Checkbox:
        return self._get_property(name=name, instance_class=Checkbox)  # type: ignore

    def get_url(self, name: str) -> Url:
        return self._get_property(name=name, instance_class=Url)  # type: ignore

    def get_number(self, name: str) -> Number:
        return self._get_property(name=name, instance_class=Number)  # type: ignore

    def get_email(self, name: str) -> Email:
        return self._get_property(name=name, instance_class=Email)  # type: ignore

    def get_phone_number(self, name: str) -> PhoneNumber:
        return self._get_property(name=name, instance_class=PhoneNumber)  # type: ignore

    def get_formula(self, name: str) -> Formula:
        return self._get_property(name=name, instance_class=Formula)  # type: ignore

    def get_rollup(self, name: str) -> Rollup:
        return self._get_property(name=name, instance_class=Rollup)  # type: ignore

    def get_unique_id(self, name: str) -> UniqueId:
        return self._get_property(name=name, instance_class=UniqueId)  # type: ignore

    def get_files(self, name: str) -> Files:
        return self._get_property(name=name, instance_class=Files)  # type: ignore

    def _get_property(self, name: str, instance_class: type) -> Property:
        result = self.properties.get_property(name=name, instance_class=instance_class)
        if result is None:
            raise NotFoundPropertyError(class_name=instance_class.__name__, prop_name=name)
        return result

    def get_parant_database_id(self) -> str | None:
        """未実装。削除すべきかも"""
        if self.parent is None or "database_id" not in self.parent:
            return None
        return self.parent["database_id"]

    def update_id_and_url(self, page_id: str, url: str) -> None:
        self.id_ = page_id
        self.url_ = url

    def title_for_slack(self) -> str:
        """Slackでの表示用のリンクつきタイトルを返す"""
        return f"<{self.url}|{self.get_title_text()}>"

    def copy(self):
        """コピーを作成する。新しいページになる"""
        cls = self.__class__
        return cls.create(properties=self.properties.values, blocks=self.block_children)

    def title_for_markdown(self) -> str:
        """Markdownでの表示用のリンクつきタイトルを返す"""
        return f"[{self.get_title_text()}]({self.url})"

    @property
    def id(self) -> str:
        if self.id_ is None:
            raise NotCreatedError("id is None.")
        return PageId(self.id_).value

    @property
    def url(self) -> str:
        if self.url_ is None:
            raise NotCreatedError("url is None.")
        return self.url_

    def is_created(self) -> bool:
        return self.id_ is not None

    def get_id_and_url(self) -> dict[str, str]:
        return {
            "id": self.id,
            "url": self.url,
        }

    @property
    def created_by(self) -> BaseOperator:
        if self._created_by is None:
            raise NotCreatedError("created_by is None.")
        return self._created_by

    @property
    def edited_by(self) -> BaseOperator:
        if self._last_edited_by is None:
            raise NotCreatedError("created_by is None.")
        return self._last_edited_by

    @classmethod
    def from_data(cls: Type[T], data: dict, block_children: list[Block] | None = None) -> T:
        id_ = PageId(data["id"]).value if data["id"] is not None else None
        url_ = data["url"] if "url" in data else None
        created_time = datetime.fromisoformat(data["created_time"]) + timedelta(hours=9)
        last_edited_time = datetime.fromisoformat(data["last_edited_time"]) + timedelta(hours=9)
        created_by = BaseOperator.of(data["created_by"])
        last_edited_by = BaseOperator.of(data["last_edited_by"])
        cover = Cover.of(data["cover"]) if data["cover"] is not None else None
        icon = Icon.of(data["icon"]) if data["icon"] is not None else None
        archived = data["archived"]
        properties = PropertyTranslator.from_dict(data["properties"])
        block_children = block_children or []

        return cls(
            id_=id_,
            url_=url_,
            created_time=created_time.replace(tzinfo=JST),
            last_edited_time=last_edited_time.replace(tzinfo=JST),
            _created_by=created_by,
            _last_edited_by=last_edited_by,
            cover=cover,
            icon=icon,
            archived=archived,
            properties=properties,
            block_children=block_children,
        )

    @classmethod
    def _get_database_id(cls) -> str:
        result = cls.DATABASE_ID
        if result is None:
            raise ValueError("DATABASE_ID is null")
        return result

    def _get_own_database_id(self) -> str:
        return self.__class__._get_database_id()



from dataclasses import dataclass
from typing import Any, Type, TypeVar

T = TypeVar("T", bound="Cover")


@dataclass
class Cover:
    type: str
    external_url: str | None = None

    def __init__(self, type: str, external_url: str | None = None) -> None:  # noqa: A002
        self.type = type
        self.external_url = external_url

    @classmethod
    def of(cls: Type[T], param: dict) -> T:
        return cls(
            type=param["type"],
            external_url=param["external"]["url"] if "external" in param else None,
        )

    @classmethod
    def from_external_url(cls: Type[T], external_url: str) -> T:
        return cls(
            type="external",
            external_url=external_url,
        )

    def __dict__(self) -> dict:
        result: dict[str, Any] = {
            "type": self.type,
        }
        if self.external_url is not None:
            result["external"] = {
                "url": self.external_url,
            }
        return result

    def value_for_filter(self) -> str:
        raise NotImplementedError



from dataclasses import dataclass
from datetime import datetime
from typing import Any, Type, TypeVar

from .property import Property
from ..datetime_utils import convert_to_date_or_datetime

T = TypeVar("T", bound="CreatedTime")


@dataclass
class CreatedTime(Property):
    def __init__(
        self,
        name: str,
        value: datetime,
        id: str | None = None,  # noqa: A002
    ) -> None:
        self.name = name
        self.value = value
        self.id = id

    @classmethod
    def create(cls: Type[T], key, value: str) -> T:
        return cls(
            name=key,
            value=convert_to_date_or_datetime(value),
        )

    def __dict__(self) -> dict[str, Any]:
        _date = {
            "start": self.value.isoformat(),
            "end": None,
            "time_zone": None,
        }
        return {
            self.name: {
                "type": self.type,
                "date": _date,
            },
        }

    @property
    def type(self) -> str:
        return "date"  # NOTE: created_timeではなくdateにする

    def value_for_filter(self) -> str:
        return self.value.isoformat()



from abc import ABCMeta, abstractmethod
from datetime import date
from typing import Any


class RichTextElement(metaclass=ABCMeta):
    annotations: dict[str, bool] | None
    plain_text: str | None
    href: dict[str, bool] | None

    def __init__(
        self,
        annotations: dict[str, bool] | None = None,
        plain_text: str | None = None,
        href: dict[str, bool] | None = None,
    ) -> None:
        self.annotations = annotations
        self.plain_text = plain_text
        self.href = href

    def to_dict(self) -> dict:
        result = {
            "type": self.get_type(),
        }
        if self.annotations is not None:
            result["annotations"] = self.annotations
        if self.plain_text is not None:
            result["plain_text"] = self.plain_text
        if self.href is not None:
            result["href"] = self.href
        result[self.get_type()] = self.to_dict_sub()
        return result

    @abstractmethod
    def to_slack_text(self) -> str:
        pass

    @staticmethod
    def from_entity(rich_text_element: dict) -> "RichTextElement":  # noqa: C901
        """dictからRichTextElementを生成する"""
        type = rich_text_element["type"]
        if type == "text":
            text = rich_text_element["text"]
            return RichTextTextElement(
                content=text["content"],
                link_url=text["link"]["url"] if text.get("link") else None,
                annotations=rich_text_element["annotations"],
                plain_text=rich_text_element["plain_text"],
                href=rich_text_element["href"],
            )
        if type == "mention":
            mention = rich_text_element["mention"]
            mention_type = mention["type"]
            if mention_type in ["database", "page"]:
                return RichTextMentionElement(
                    mention_type=mention_type,
                    object_id=mention[mention_type]["id"],
                    annotations=rich_text_element["annotations"],
                    plain_text=rich_text_element["plain_text"],
                    href=rich_text_element["href"],
                )
            if mention_type == "date":
                return RichTextMentionElement(
                    mention_type=mention_type,
                    start_date=mention["date"]["start"],
                    end_date=mention["date"]["end"],
                    annotations=rich_text_element["annotations"],
                    plain_text=rich_text_element["plain_text"],
                    href=rich_text_element["href"],
                )
            if mention_type == "link_preview":
                return RichTextMentionElement(
                    mention_type=mention_type,
                    link_preview_url=mention["link_preview"]["url"],
                    annotations=rich_text_element["annotations"],
                    plain_text=rich_text_element["plain_text"],
                    href=rich_text_element["href"],
                )
            raise Exception("invalid mention type")
        if type == "equation":
            raise NotImplementedError("equation is not implemented yet")
        raise Exception("invalid type")

    @abstractmethod
    def to_plain_text(self) -> str:
        """plain_textに変換する"""

    @abstractmethod
    def get_type(self) -> str:
        """text, mention, equationのどれかを返す"""

    @abstractmethod
    def to_dict_sub(self) -> str:
        """Text, Mention, Equationのそれぞれで実装する"""


class RichTextTextElement(RichTextElement):
    content: str
    link_url: str | None = None

    def __init__(
        self,
        content: str,
        link_url: str | None = None,
        annotations: dict[str, bool] | None = None,
        plain_text: str | None = None,
        href: dict[str, bool] | None = None,
    ) -> None:
        self.content = content
        self.link_url = link_url
        super().__init__(annotations, plain_text, href)

    @staticmethod
    def of(content: str, link_url: str | None = None) -> "RichTextTextElement":
        return RichTextTextElement(
            content=content,
            link_url=link_url,
        )

    def to_slack_text(self) -> str:
        return self.content

    def to_plain_text(self) -> str:
        return self.content

    def get_type(self) -> str:
        return "text"

    def to_dict_sub(self) -> str:
        result = {
            "content": self.content,
        }
        if self.link_url is not None:
            result["link"] = {
                "url": self.link_url,
            }
        return result


class RichTextMentionElement(RichTextElement):
    # TODO: 日付やリンクプレビューなどもあるみたい
    # refs: https://developers.notion.com/reference/rich-text#mention
    mention_type: str  # database, page, date, link_preview
    object_id: str | None  # database,pageのどちらかで利用。database_idまたはpage_id
    start_date: str | None = None  # dateのみ利用
    end_date: str | None = None  # dateのみ利用
    link_preview_url: str | None = None  # link_previewのみ利用

    def __init__(
        self,
        mention_type: str,
        object_id: str | None = None,
        start_date: str | None = None,
        end_date: str | None = None,
        link_preview_url: str | None = None,
        annotations: dict[str, bool] | None = None,
        plain_text: str | None = None,
        href: dict[str, bool] | None = None,
    ) -> None:
        self.mention_type = mention_type
        self.object_id = object_id
        self.start_date = start_date
        self.end_date = end_date
        self.link_preview_url = link_preview_url
        super().__init__(annotations, plain_text, href)

    @classmethod
    def from_page_type(cls: "RichTextMentionElement", page_id: str) -> "RichTextMentionElement":
        return cls(mention_type="page", object_id=page_id)

    def to_slack_text(self) -> str:
        # TODO: メンションの場合、そのタイトルは再度クライアントを使ってretrieveしないといけない
        # これをどうするか迷うけど、とりあえず空文字で返す
        return self.to_plain_text()

    def to_plain_text(self) -> str:
        return self.plain_text if self.plain_text is not None else ""

    @staticmethod
    def of_database(database_id: str) -> "RichTextMentionElement":
        return RichTextMentionElement(
            mention_type="database",
            object_id=database_id,
        )

    @staticmethod
    def of_page(page_id: str) -> "RichTextMentionElement":
        return RichTextMentionElement(
            mention_type="page",
            object_id=page_id,
        )

    @staticmethod
    def of_date(start: date, end: date | None = None) -> "RichTextMentionElement":
        return RichTextMentionElement(
            mention_type="date",
            start_date=start.isoformat(),
            end_date=end.isoformat() if end is not None else None,
        )

    def get_type(self) -> str:
        return "mention"

    def to_dict_sub(self) -> dict:
        """Text, Mention, Equationのそれぞれで実装する"""
        result: dict[str, Any] = {
            "type": self.mention_type,
        }
        if self.mention_type in ["database", "page"]:
            result[self.mention_type] = {
                "id": self.object_id,
            }
        if self.mention_type == "date":
            result["date"] = {
                "start": self.start_date,
            }
            if self.end_date is not None:
                result["date"]["end"] = self.end_date

        return result


class RichTextEquationElement(RichTextElement):
    # TODO: not implemented yet
    pass



from .block import Block
from .rich_text.rich_text import RichText


class Image(Block):
    image_caption: list
    image_type: str
    image_file: dict | None
    image_external: dict | None
    type: str = "image"

    def __init__(
        self,
        image_caption: list,
        image_type: str,
        image_file: dict | None = None,
        image_external: dict | None = None,
        id: str | None = None,  # noqa: A002
        archived: bool | None = None,
        created_time: str | None = None,
        last_edited_time: str | None = None,
        has_children: bool | None = None,
        parent: dict | None = None,
    ):
        super().__init__(
            id, archived, created_time, last_edited_time, has_children, parent
        )
        self.image_caption = image_caption
        self.image_type = image_type
        self.image_file = image_file
        self.image_external = image_external

    @staticmethod
    def of(block: dict) -> "Image":
        image = block["image"]
        image_caption = image.get("caption", [])
        image_type = image.get("type", "")
        image_file = image.get("file")
        image_external = image.get("external")
        return Image(
            id=block["id"],
            archived=block["archived"],
            created_time=block["created_time"],
            last_edited_time=block["last_edited_time"],
            has_children=block["has_children"],
            parent=block["parent"],
            image_caption=image_caption,
            image_type=image_type,
            image_file=image_file,
            image_external=image_external,
        )

    def to_dict_sub(self) -> dict:
        if self.image_type == "file":
            msg = "fileタイプは未実装"
            raise NotImplementedError(msg)
        if self.image_type == "external":
            return {
                "caption": self.image_caption,
                "type": self.image_type,
                "external": self.image_external,
            }
        msg = f"Invalid image type: {self.image_type}"
        raise ValueError(msg)

    def to_slack_text(self) -> str:
        if self.image_type == "file":
            return self.image_file["url"]
        if self.image_type == "external":
            return self.image_external["url"]
        msg = f"Invalid image type: {self.image_type}"
        raise ValueError(msg)

    @classmethod
    def from_external_url(cls, url: str, alias_url: str | None = None) -> "Image":
        image_caption = []
        if alias_url:
            image_caption = RichText.from_plain_link(alias_url, alias_url).to_dict()
        return Image(
            image_caption=image_caption,
            image_type="external",
            image_external={"url": url},
        )



import re
from dataclasses import dataclass
from types import NoneType
from typing import Any

from ..page.page_id import PageId
from .condition.cond import Cond
from .condition.prop import Prop

RULESET: dict[Prop, dict[Cond, list[type]]] = {}
RULESET[Prop.RICH_TEXT] = {
    Cond.EQUALS: [str],
    Cond.DOES_NOT_EQUAL: [str],
    Cond.CONTAINS: [str],
    Cond.DOES_NOT_CONTAIN: [str],
    Cond.IS_EMPTY: [bool],
    Cond.IS_NOT_EMPTY: [bool],
    Cond.STARTS_WITH: [str],
    Cond.ENDS_WITH: [str],
}
RULESET[Prop.CHECKBOX] = {
    Cond.EQUALS: [bool],
    Cond.DOES_NOT_EQUAL: [bool],
}
RULESET[Prop.DATE] = {
    Cond.EQUALS: [str],
    Cond.AFTER: [str],
    Cond.ON_OR_AFTER: [str],
    Cond.BEFORE: [str],
    Cond.ON_OR_BEFORE: [str],
    Cond.IS_EMPTY: [bool],
    Cond.IS_NOT_EMPTY: [bool],
    Cond.NEXT_WEEK: [NoneType],
    Cond.NEXT_MONTH: [NoneType],
    Cond.NEXT_YEAR: [NoneType],
    Cond.PAST_WEEK: [NoneType],
    Cond.PAST_MONTH: [NoneType],
    Cond.PAST_YEAR: [NoneType],
    Cond.THIS_WEEK: [NoneType],
}
RULESET[Prop.FILES] = {
    Cond.IS_EMPTY: [bool],
    Cond.IS_NOT_EMPTY: [bool],
}
RULESET[Prop.MULTI_SELECT] = {
    Cond.CONTAINS: [str],
    Cond.DOES_NOT_CONTAIN: [str],
    Cond.IS_EMPTY: [bool],
    Cond.IS_NOT_EMPTY: [bool],
}
RULESET[Prop.NUMBER] = {
    Cond.EQUALS: [int, float],
    Cond.DOES_NOT_EQUAL: [int, float],
    Cond.GREATER_THAN: [int, float],
    Cond.LESS_THAN: [int, float],
    Cond.GREATER_THAN_OR_EQUAL_TO: [int, float],
    Cond.LESS_THAN_OR_EQUAL_TO: [int, float],
    Cond.IS_EMPTY: [bool],
    Cond.IS_NOT_EMPTY: [bool],
}
RULESET[Prop.PEOPLE] = {
    Cond.CONTAINS: [str],
    Cond.DOES_NOT_CONTAIN: [str],
    Cond.IS_EMPTY: [bool],
    Cond.IS_NOT_EMPTY: [bool],
}
RULESET[Prop.RELATION] = {
    Cond.CONTAINS: [str],
    Cond.DOES_NOT_CONTAIN: [str],
    Cond.IS_EMPTY: [bool],
    Cond.IS_NOT_EMPTY: [bool],
}
RULESET[Prop.SELECT] = {
    Cond.EQUALS: [str],
    Cond.DOES_NOT_EQUAL: [str],
    Cond.IS_EMPTY: [bool],
    Cond.IS_NOT_EMPTY: [bool],
}
RULESET[Prop.STATUS] = {
    Cond.EQUALS: [str],
    Cond.DOES_NOT_EQUAL: [str],
    Cond.IS_EMPTY: [bool],
    Cond.IS_NOT_EMPTY: [bool],
}
RULESET[Prop.ID] = {
    Cond.EQUALS: [int],
    Cond.DOES_NOT_EQUAL: [int],
    Cond.GREATER_THAN: [int],
    Cond.LESS_THAN: [int],
    Cond.GREATER_THAN_OR_EQUAL_TO: [int],
    Cond.LESS_THAN_OR_EQUAL_TO: [int],
    Cond.IS_EMPTY: [bool],
    Cond.IS_NOT_EMPTY: [bool],
}
RULESET[Prop.CREATED_TIME] = {
    Cond.EQUALS: [str],
    Cond.AFTER: [str],
    Cond.ON_OR_AFTER: [str],
    Cond.BEFORE: [str],
    Cond.ON_OR_BEFORE: [str],
    Cond.IS_EMPTY: [bool],
    Cond.IS_NOT_EMPTY: [bool],
    Cond.NEXT_WEEK: [NoneType],
    Cond.NEXT_MONTH: [NoneType],
    Cond.NEXT_YEAR: [NoneType],
    Cond.PAST_WEEK: [NoneType],
    Cond.PAST_MONTH: [NoneType],
    Cond.PAST_YEAR: [NoneType],
    Cond.THIS_WEEK: [NoneType],
}
RULESET[Prop.LAST_EDITED_TIME] = {
    Cond.EQUALS: [str],
    Cond.AFTER: [str],
    Cond.ON_OR_AFTER: [str],
    Cond.BEFORE: [str],
    Cond.ON_OR_BEFORE: [str],
    Cond.IS_EMPTY: [bool],
    Cond.IS_NOT_EMPTY: [bool],
    Cond.NEXT_WEEK: [NoneType],
    Cond.NEXT_MONTH: [NoneType],
    Cond.NEXT_YEAR: [NoneType],
    Cond.PAST_WEEK: [NoneType],
    Cond.PAST_MONTH: [NoneType],
    Cond.PAST_YEAR: [NoneType],
    Cond.THIS_WEEK: [NoneType],
}


@dataclass(frozen=True)
class ConditionRuleset:
    prop: Prop
    cond: Cond
    value: Any

    def validate(self) -> None:
        # 必須のチェック
        self.validate_prop()
        self.validate_cond()
        self.validate_value()

        # オプションのチェック
        self.validate_page_id()
        self.validate_date()

    def validate_prop(self) -> None:
        if self.prop not in RULESET:
            raise ValueError(f"Property {self.prop} is not supported")

    def validate_cond(self) -> None:
        if self.cond not in RULESET[self.prop]:
            msg = f"Condition {self.cond} is not supported for property {self.prop}"
            raise ValueError(msg)

    def validate_value(self) -> None:
        if type(self.value) not in RULESET[self.prop][self.cond]:
            msg = f"Value type {type(self.value)} is not supported for property {self.prop} with condition {self.cond}"
            raise ValueError(msg)

    def validate_page_id(self) -> None:
        if self.prop not in [Prop.PEOPLE, Prop.RELATION]:
            return
        if self.cond not in [Cond.CONTAINS, Cond.DOES_NOT_CONTAIN]:
            return
        # ユーザもしくはリレーションの場合は、ページIDに変換できるかどうかを確認する
        PageId(self.value)

    def validate_date(self) -> None:
        if self.prop != Prop.DATE:
            return
        if self.cond not in [
            Cond.AFTER,
            Cond.ON_OR_AFTER,
            Cond.BEFORE,
            Cond.ON_OR_BEFORE,
        ]:
            return
        # "2021-05-10"、"2021-05-10T12:00:00"、"2021-10-15T12:00:00-07:00"
        # のような形式であるかどうかを正規表現で確認する
        if re.match(r"\d{4}-\d{2}-\d{2}", self.value):
            return
        if re.match(r"\d{4}-\d{2}-\d{2}T?\d{2}:\d{2}:\d{2}", self.value):
            return
        if re.match(r"\d{4}-\d{2}-\d{2}T?\d{2}:\d{2}:\d{2}[-+]?\d{2}:\d{2}", self.value):
            return
        raise ValueError(f"Date value {self.value} is invalid")
